@@ -1,5 +1,5 @@
 package Consul;
-$Consul::VERSION = '0.005';
+$Consul::VERSION = '0.006';
 # ABSTRACT: Client library for consul
 
 use namespace::autoclean;
@@ -8,6 +8,7 @@ use HTTP::Tiny 0.014;
 use URI::Escape qw(uri_escape);
 use JSON::MaybeXS qw(JSON);
 use Hash::MultiValue;
+use Try::Tiny;
 use Carp qw(croak);
 
 use Moo;
@@ -44,22 +45,26 @@ my $json = JSON->new->utf8->allow_nonref;
 sub _prep_request {
     my ($self, $path, $method, %args) = @_;
 
-    my $content = delete $args{_content};
-    delete $args{$_} for grep { m/^_/ } keys %args;
+    my %uargs = map { m/^_/ ? () : ($_ => $args{$_}) } keys %args;
 
     my $headers = Hash::MultiValue->new;
 
-    return ($method, $self->_prep_url($path, %args), $headers, $content);
+    return ($method, $self->_prep_url($path, %uargs), $headers, $args{_content});
 }
 
 sub _prep_response {
-    my ($self, $status, $reason, $headers, $content) = @_;
+    my ($self, $status, $reason, $headers, $content, %args) = @_;
 
-    croak "$status $reason: $content" unless int($status/100) == 2;
+    my $valid_cb = $args{_valid_cb} // sub { int($_[0]/100) == 2 };
 
-    return if !defined $content || length $content == 0;
+    croak "$status $reason: $content" unless $valid_cb->($status);
 
-    return $json->decode($content);
+    my $data;
+    $data = $json->decode($content) if defined $content && length $content > 0;
+
+    my $meta = try { Consul::Meta->new(%$headers) };
+
+    return ($data, $meta);
 }
 
 has req_cb => ( is => 'lazy', isa => CodeRef );
@@ -80,12 +85,15 @@ sub _api_exec {
     my $resp_cb = $#_ % 2 == 1 && ref $_[$#_] eq 'CODE' ? pop @_ : sub { pop @_ };
     my ($self, $path, $method, %args) = @_;
 
-    my $r;
-    my $cli_cb = delete $args{cb} // sub { $r = shift };
+    my @r;
+    my $cli_cb = delete $args{cb} // sub { @r = @_ };
 
-    $self->req_cb->($self, $self->_prep_request($path, $method, %args), sub { $cli_cb->($resp_cb->($self->_prep_response(@_))) });
+    $self->req_cb->($self, $self->_prep_request($path, $method, %args), sub {
+        my ($data, $meta) = $self->_prep_response(@_, %args);
+        $cli_cb->($resp_cb->($data), $meta);
+    });
 
-    return $r;
+    return wantarray ? @r : shift @r;
 };
 
 with qw(
@@ -102,6 +110,18 @@ with qw(
 use Consul::Check;
 use Consul::Service;
 
+
+package
+    Consul::Meta; # hide from PAUSE
+
+use Moo;
+use Types::Standard qw(Int Bool);
+
+has index        => ( is => 'ro', isa => Int,  init_arg => 'x-consul-index',       required => 1 );
+has last_contact => ( is => 'ro', isa => Int,  init_arg => 'x-consul-lastcontact', required => 1 );
+has known_leader => ( is => 'ro', isa => Bool, init_arg => 'x-consul-knownleader', required => 1, coerce => sub { { true => 1, false => 0 }->{$_[0]} // $_[0] } );
+
+
 1;
 
 =pod
@@ -272,6 +292,19 @@ cluster, you can ignore this option entirely.
 
 =back
 
+=head1 BLOCKING QUERIES
+
+Some Consul API endpoints support a feature called a "blocking query". These
+endpoints allow long-polling for changes, and support some extra information
+about the server state, including the Raft index, in the response headers.
+
+The corresponding endpoint methods, when called in array context, will return a
+second value. This is an object with three methods, C<index>, C<last_contact>
+and C<known_leader>, corresponding to the similarly-named header fields. You
+can use these to set up state watches, CAS writes, and so on.
+
+See the Consul API docs for more information.
+
 =head1 SEE ALSO
 
 =over 4
