@@ -1,5 +1,5 @@
 package Consul::API::Event;
-$Consul::API::Event::VERSION = '0.005';
+$Consul::API::Event::VERSION = '0.006';
 use namespace::autoclean;
 
 use Moo::Role;
@@ -29,7 +29,9 @@ sub fire {
     my ($self, $name, %args) = @_;
     croak 'usage: $event->fire($name, [%args])' if grep { !defined } ($name);
     my $payload = delete $args{payload};
-    Consul::API::Event::Event->new($$self->_api_exec($$self->_event_endpoint."/fire/".$name, 'PUT', %args, ($payload ? (_content => $payload) : ())));
+    $$self->_api_exec($$self->_event_endpoint."/fire/".$name, 'PUT', %args, ($payload ? (_content => $payload) : ()), sub {
+        Consul::API::Event::Event->new($_[0])
+    });
 }
 
 sub list {
@@ -40,7 +42,7 @@ sub list {
 }
 
 package Consul::API::Event::Event;
-$Consul::API::Event::Event::VERSION = '0.005';
+$Consul::API::Event::Event::VERSION = '0.006';
 use Convert::Base64 qw(decode_base64);
 
 use Moo;
